import re
import urllib2
from bs4 import BeautifulSoup
import json
import tempfile
import os
import imp
import webbrowser
from helper import data_dict, template_string


"""
A web spawler to extract data from Leetcode

"""
class LeetCode:
    def __init__(self):
        self.temp_dir = tempfile.mkdtemp()
        self.data = []
        self.js_lib_path = imp.find_module('leetcodepv')[1]

    def get_tags(self):
        page = urllib2.urlopen(data_dict['ORIGINAL_URL']).read()
        soup = BeautifulSoup(page)
        tags = []
        for x in soup.find_all('a'):
            current = x.get('href')
            if current and 'tag' in current:
                tags.append(current)
        return tags

    def get_urls_for_tag(self, tags):
        ans = {}
        for tag in tags:
            _tag = tag.split('/')[2].replace('-', ' ')
            res, input_url = '', data_dict['PREFIX'] + tag
            page = urllib2.urlopen(input_url).read()
            for x in BeautifulSoup(page).find_all('script'):
                current = x.get_text().strip()
                if 'title' in current:
                    res += current
            for x in BeautifulSoup(res).find_all('a'):
                name = str(x.get_text())
                if name in ans:
                    ans[name] += [_tag]
                else:
                    ans[name] = [_tag]
        return {x: self.categorize_problems(ans[x])for x in ans}

    def read_oj(self, url):
        page = urllib2.urlopen(url).read()
        soup = BeautifulSoup(page)
        table = soup.find("table", attrs={"id": "problemList"})
        ans, rowdata = [], []
        for row in table.find_all("td"):
            current = row.get_text().strip()
            if current:
                rowdata.append(current)
            if len(rowdata) == 4:
                no, name, difficulty = int(rowdata[0]), str(rowdata[1]), str(rowdata[3])
                url = ''
                if no not in data_dict['NOT_FREE']:
                    url = '{}/problems/{}'\
                        .format(data_dict['PREFIX'], \
                        re.sub("[^a-zA-Z0-9\s\-]", "", name).replace(' ', '-'))
                ans.append([no, name, difficulty, url])
                rowdata = []
        return {x[1]: x[2] for x in ans}

    def categorize_problems(self, arr):
        dtst, algo = [], []
        for x in arr:
            if x in data_dict['DATA_STRUCTURE']:
                dtst.append(x.title())
            else:
                algo.append(x.title())
        return map(self.trans_arr, [dtst, algo])

    def trans_arr(self, arr):
        n = len(arr)
        if n == 0:
            return 'N/A'
        if n == 1:
            return arr[0]
        if n > 1:
            arr.sort()
            return ' + '.join(arr)

    def combine_tag_url(self, tagdata, oj_urls):
        ans = []
        for x in tagdata:
            name, dtst, algo, level = x, tagdata[x][0], tagdata[x][1], oj_urls[x]
            current = {"Name": name, "Data Structure": dtst, "Algorithm": algo, \
                "Level": level}
            ans.append(current)
        return ans

    def get_data(self):
        tags = self.get_tags()
        tagdata = self.get_urls_for_tag(tags)
        oj_urls = self.read_oj(data_dict['ORIGINAL_URL'])
        self.data = self.combine_tag_url(tagdata, oj_urls)

    def create_json(self):
        self.get_data()
        with open(os.path.join(self.temp_dir, 'leetcode.json'), 'w') as outfile:
            json.dump(self.data, outfile)

    def create_html(self):
        js_urls = data_dict['JS_URLS']
        x = template_string
        for i in range(4):
            x = x.replace('#REPLACE{}'.format(i), \
                os.path.join(self.js_lib_path, 'ext', js_urls[i]))
        destiny = os.path.join(self.temp_dir, 'leetcode.html')
        with open(destiny, 'w') as outfile:
            outfile.write(x)
        return destiny

    def view_pt(self):
        self.create_json()
        webbrowser.open_new(self.create_html())


